/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	"github.com/rancher/wrangler/pkg/generic"
	v1 "k8s.io/api/rbac/v1"
	"k8s.io/apimachinery/pkg/api/equality"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	informers "k8s.io/client-go/informers/rbac/v1"
	clientset "k8s.io/client-go/kubernetes/typed/rbac/v1"
	listers "k8s.io/client-go/listers/rbac/v1"
	"k8s.io/client-go/tools/cache"
)

type RoleHandler func(string, *v1.Role) (*v1.Role, error)

type RoleController interface {
	generic.ControllerMeta
	RoleClient

	OnChange(ctx context.Context, name string, sync RoleHandler)
	OnRemove(ctx context.Context, name string, sync RoleHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() RoleCache
}

type RoleClient interface {
	Create(*v1.Role) (*v1.Role, error)
	Update(*v1.Role) (*v1.Role, error)

	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1.Role, error)
	List(namespace string, opts metav1.ListOptions) (*v1.RoleList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.Role, err error)
}

type RoleCache interface {
	Get(namespace, name string) (*v1.Role, error)
	List(namespace string, selector labels.Selector) ([]*v1.Role, error)

	AddIndexer(indexName string, indexer RoleIndexer)
	GetByIndex(indexName, key string) ([]*v1.Role, error)
}

type RoleIndexer func(obj *v1.Role) ([]string, error)

type roleController struct {
	controllerManager *generic.ControllerManager
	clientGetter      clientset.RolesGetter
	informer          informers.RoleInformer
	gvk               schema.GroupVersionKind
}

func NewRoleController(gvk schema.GroupVersionKind, controllerManager *generic.ControllerManager, clientGetter clientset.RolesGetter, informer informers.RoleInformer) RoleController {
	return &roleController{
		controllerManager: controllerManager,
		clientGetter:      clientGetter,
		informer:          informer,
		gvk:               gvk,
	}
}

func FromRoleHandlerToHandler(sync RoleHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.Role
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.Role))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *roleController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.Role))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateRoleDeepCopyOnChange(client RoleClient, obj *v1.Role, handler func(obj *v1.Role) (*v1.Role, error)) (*v1.Role, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *roleController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, handler)
}

func (c *roleController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), handler)
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, removeHandler)
}

func (c *roleController) OnChange(ctx context.Context, name string, sync RoleHandler) {
	c.AddGenericHandler(ctx, name, FromRoleHandlerToHandler(sync))
}

func (c *roleController) OnRemove(ctx context.Context, name string, sync RoleHandler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), FromRoleHandlerToHandler(sync))
	c.AddGenericHandler(ctx, name, removeHandler)
}

func (c *roleController) Enqueue(namespace, name string) {
	c.controllerManager.Enqueue(c.gvk, c.informer.Informer(), namespace, name)
}

func (c *roleController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controllerManager.EnqueueAfter(c.gvk, c.informer.Informer(), namespace, name, duration)
}

func (c *roleController) Informer() cache.SharedIndexInformer {
	return c.informer.Informer()
}

func (c *roleController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *roleController) Cache() RoleCache {
	return &roleCache{
		lister:  c.informer.Lister(),
		indexer: c.informer.Informer().GetIndexer(),
	}
}

func (c *roleController) Create(obj *v1.Role) (*v1.Role, error) {
	return c.clientGetter.Roles(obj.Namespace).Create(context.TODO(), obj, metav1.CreateOptions{})
}

func (c *roleController) Update(obj *v1.Role) (*v1.Role, error) {
	return c.clientGetter.Roles(obj.Namespace).Update(context.TODO(), obj, metav1.UpdateOptions{})
}

func (c *roleController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.clientGetter.Roles(namespace).Delete(context.TODO(), name, *options)
}

func (c *roleController) Get(namespace, name string, options metav1.GetOptions) (*v1.Role, error) {
	return c.clientGetter.Roles(namespace).Get(context.TODO(), name, options)
}

func (c *roleController) List(namespace string, opts metav1.ListOptions) (*v1.RoleList, error) {
	return c.clientGetter.Roles(namespace).List(context.TODO(), opts)
}

func (c *roleController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.clientGetter.Roles(namespace).Watch(context.TODO(), opts)
}

func (c *roleController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.Role, err error) {
	return c.clientGetter.Roles(namespace).Patch(context.TODO(), name, pt, data, metav1.PatchOptions{}, subresources...)
}

type roleCache struct {
	lister  listers.RoleLister
	indexer cache.Indexer
}

func (c *roleCache) Get(namespace, name string) (*v1.Role, error) {
	return c.lister.Roles(namespace).Get(name)
}

func (c *roleCache) List(namespace string, selector labels.Selector) ([]*v1.Role, error) {
	return c.lister.Roles(namespace).List(selector)
}

func (c *roleCache) AddIndexer(indexName string, indexer RoleIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.Role))
		},
	}))
}

func (c *roleCache) GetByIndex(indexName, key string) (result []*v1.Role, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v1.Role, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v1.Role))
	}
	return result, nil
}
